"""
Mixin class for time registration
"""

# python
from types import StringTypes
# zope
from AccessControl import ClassSecurityInfo, Permissions
# cmf
from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.utils import getToolByName


class TimeRegistrationMixin:


    security = ClassSecurityInfo()


    security.declareProtected(View, 'getTimeRegistrations')
    def getTimeRegistrations(self):
        "Returns time registrations from all tasks"
        tasks = self.getTasks()
        result = []
        for task in tasks:
            result += task.getTimeRegistrationsWithTaskTitle().values()
        return result


    security.declarePublic('rangeFilter')
    def rangeFilter(self, regs):
        "Filters a list of tasks for range validity"
        drd = self.getDateRangeDict()
        start = self.ZopeTime(drd['start_year'], drd['start_month'], drd['start_day'])
        end = self.ZopeTime(drd['end_year'], drd['end_month'], drd['end_day'])
        result = []
        for reg in regs:
            if not drd['filter_on'] or (drd['filter_on'] and (start <= reg['logtime'] <= end)):
                result.append(reg)
        return result
        

    security.declareProtected(View, 'getResolvedRegistrations')
    def getResolvedRegistrations(self):
        """
        Do *not* use this for modifications. This is a user interface helper.
        Returns list of all the time registrations. Not dict!. Fullname and email are 
        resolved. id is inserted also.
        Also sorts by key, if sortkey is set in the request.
        """
        regs = self.getTimeRegistrations()
        result = self.rangeFilter(regs)
        for reg in result:
            reg['id'] = '' # not used
            memberId = reg.get('memberId', '')
            memberProps = self.getMemberProperties(memberId, ['fullname', 'email'])
            reg['fullname'], reg['email'] = memberProps
        # sort
        sortkey = self.REQUEST.get('sortkey', 'logtime')
        decorated = []
        for r in result:
            sortval = r.get(sortkey, '')
            if hasattr(sortval, 'lower'):
                sortval = sortval.lower()
            decorated.append((sortval, r))
        decorated.sort()
        result = [d[-1] for d in decorated]
        return result


    security.declarePrivate('getMemberProperties')
    def getMemberProperties(self, memberId, properties):
        "Returns one or more member properties"
        if isinstance(properties, StringTypes):
            properties = [properties]
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getMemberById(memberId)
        if member is None: # just return empty strings if member has been deleted
            vals = ['']*len(properties)
        else:
            vals = [member.getProperty(property, '') for property in properties]
        if len(vals) == 1:
            return vals[0]
        else:
            return vals


    security.declareProtected(View, 'getMemberFullname')
    def getMemberFullname(self, memberId):
        "Returns a members full name"
        return self.getMemberProperties(memberId, 'fullname')


    security.declareProtected(View, 'getTimeRegsByKeyGroup')
    def getTimeRegsByKeyGroup(self, key, regs):
        """
        Returns time regs as dict with list of objects, by key. Eg:
        {'memberId1':[reg1, reg2, reg3], {'memberId2':[reg4, reg5, reg6]}
        """
        result = {}
        for reg in regs:
            result.setdefault(reg[key], []).append(reg)
        return result


    security.declareProtected(View, 'getTimeByKeyGroup')
    def getTimeByKeyGroup(self, key, regs):
        """
        Returns the time in keygroups. eg if key=='attendeeId'
        {'attendee1':42.0, 'attendee2':24.0}
        """
        regGroups = self.getTimeRegsByKeyGroup(key, regs)
        result = {}
        for k, registrations in regGroups.items():
            result[k] = self.getTotalTime(registrations)
        return result


    security.declareProtected(View, 'getTimeByKeyGroup')
    def getTotalTime(self, regs):
        """
        Returns the total time.
        """
        return sum([float(reg.get('time', 0.0)) for reg in regs])


    # milestone methods
    
    def getMilestones(self):
        pass